@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity preferences */
 
 /* 
@@ -257,6 +259,7 @@ typedef enum _MetaKeyBindingAction
 typedef struct
 {
   unsigned int keysym;
+  unsigned int keycode;
   MetaVirtualModifier modifiers;
 } MetaKeyCombo;
 
